/*
 * Somewhat naive implementation of a parser for a text file
 * with an email message defined by following grammar:
 * <message>    ::= <sender> <recipient>+ <carboncopy>* <subject> <body>
 * <sender>     ::= "from:" <address>
 * <recipient>  ::= "to:" <address>
 * <carboncopy> ::= "cc:" <address>
 * <subject>    ::= "subject:" <string>+
 * <body>       ::= "body:" <string>*
 * <address>    ::= <id>'@'<id>
 * <id>         ::= ['a'..'z','A'..'Z','0'..'9','.','-','_']+
 * <string>     ::= ['a'..'z','A'..'Z','0'..'9',''..'','.',',','_']+
 * 
 * (assuming that <id> and <string> do not contain key tokens)
 */

import java.util.*;
import java.io.*;

public class SimpleMailer {
	private Set keys;
	private Message msg;
	
	public SimpleMailer() {
		keys = new HashSet();
		keys.add("from:");	
		keys.add("to:");	
		keys.add("cc:");	
		keys.add("subject:");	
		keys.add("body:");	
	}
		
	public static void main(String[] args) {
		if (args.length != 1) {
			System.out.println("Usage: java SimpleMailer <message file>");
			System.exit(0);
		}
		SimpleMailer mailer = new SimpleMailer();
		mailer.processMessage(args[0]);
	}

	private void processMessage(String fileName) {
		String buffer = new String();
		try {
			BufferedReader reader = new BufferedReader(new FileReader(fileName));
			while (reader.ready()) {
				buffer += reader.readLine() + " ";	
			}
			reader.close();
		}
		catch (IOException ex) {
			ex.printStackTrace();
			System.exit(1);
		}
		parseMessage(buffer);
		System.out.println("*** Message ***\n" + msg);
	}
	
	private void parseMessage(String str) {
		msg = new Message();
		StringTokenizer tokenizer = new StringTokenizer(str);
		String token = tokenizer.nextToken();
		// Process single sender
		if (token.equals("from:")) {
			String tmp = new String();
			token = tokenizer.nextToken();
			while (tokenizer.hasMoreTokens() && !keys.contains(token)) {
				tmp += token + " ";
				token = tokenizer.nextToken();
			}
			processSender(tmp);
		} else {
			System.out.println("-> Expected sender!");
		}
		// Process multiple recipients
		if (token.equals("to:")) {
			while (token.equals("to:")) {
				String tmp = new String();
				token = tokenizer.nextToken();
				while (tokenizer.hasMoreTokens() && !keys.contains(token)) {
					tmp += token + " ";
					token = tokenizer.nextToken();
				}
				processRecipient(tmp);				
			}
		} else {
			System.out.println("-> Expected recipient!");
		}
		// Process multiple carbon copies if present
		while (token.equals("cc:")) {
			String tmp = new String();
			token = tokenizer.nextToken();
			while (tokenizer.hasMoreTokens() && !keys.contains(token)) {
				tmp += token + " ";
				token = tokenizer.nextToken();
			}
			processCarbonCopy(tmp);				
		}
		// Process single subject
		if (token.equals("subject:")) {
			String tmp = new String();
			token = tokenizer.nextToken();
			while (tokenizer.hasMoreTokens() && !keys.contains(token)) {
				tmp += token + " ";
				token = tokenizer.nextToken();
			}
			if (tmp.length() > 0)
				msg.setSubject(tmp);
			else
				System.out.println("-> Empty subject!");				
		} else {
			System.out.println("-> Expected subject!");
		}
		// Process single body
		if (token.equals("body:")) {
			String tmp = new String();
			while (tokenizer.hasMoreTokens()) {
				token = tokenizer.nextToken();
				tmp += token + " ";
			}
			msg.setBody(tmp);				
		} else {
			System.out.println("-> Expected body!");
		}
	}

	private void processSender(String str) {
		StringTokenizer tokenizer = new StringTokenizer(str);
		if (tokenizer.countTokens() != 1)
			System.out.println("-> Not exactly one sender address!");
		if (tokenizer.hasMoreTokens()) {
			String adr = tokenizer.nextToken();
			checkAddress(adr);
			msg.setSender(adr);
		}
	}
	
	private void processRecipient(String str) {
		StringTokenizer tokenizer = new StringTokenizer(str);
		if (tokenizer.countTokens() != 1)
			System.out.println("-> Not exactly one recipient address!");
		if (tokenizer.hasMoreTokens()) {
			String adr = tokenizer.nextToken();
			checkAddress(adr);
			msg.getRecipients().add(adr);
		}
	}
	
	private void processCarbonCopy(String str) {
		StringTokenizer tokenizer = new StringTokenizer(str);
		if (tokenizer.countTokens() != 1)
			System.out.println("-> Not exactly one carbon copy address!");
		if (tokenizer.hasMoreTokens()) {
			String adr = tokenizer.nextToken();
			checkAddress(adr);
			msg.getCarbonCopies().add(adr);
		}
	}
		
	private boolean checkAddress(String adr) {
		StringTokenizer tokenizer = new StringTokenizer(adr, "@");
		if (tokenizer.countTokens() != 2) {
			System.out.println("-> Invalid address: \"" + adr + "\"!");
			return false;
		}
		return true;
	}
}
